@@ -28,7 +28,9 @@ class const:  # noqa
     ERROR_OCCURRED = 'ERROR_OCCURRED'
 
     BEFORE_START = 'BEFORE_START'
+    BEFORE_WORKER_START = 'BEFORE_WORKER_START'
     BEFORE_CLOSE = 'BEFORE_CLOSE'
+    BEFORE_WORKER_CLOSE = 'BEFORE_WORKER_CLOSE'
 
     ON_CONSUME = 'ON_CONSUME'
 
