@@ -19,11 +19,21 @@ import sys
 import threading
 from typing import Any, List, Union, Mapping, Callable, Literal, Tuple
 
+import better_exceptions
 from loguru import logger
 
 JSONP_REGEX = re.compile(r'\S+?\((?P<obj>[\s\S]*)\)')
 PACKAGE_REGEX = re.compile(r"([a-zA-Z0-9_\-]+)([<>=]*)([\d.]*)")
 
+better_exceptions.MAX_LENGTH = None
+exec_formatter = better_exceptions.ExceptionFormatter(
+    colored=False,
+    theme=better_exceptions.THEME,
+    max_length=better_exceptions.MAX_LENGTH,
+    pipe_char=better_exceptions.PIPE_CHAR,
+    cap_char=better_exceptions.CAP_CHAR
+)
+
 
 def load_objects(path_or_reference, reload=False):
     """
@@ -98,14 +108,16 @@ def require(
             return installed_version
 
     except importlib_metadata.PackageNotFoundError:
-        if action == "raise":
-            raise importlib_metadata.PackageNotFoundError(f"缺少依赖包: {package_spec}")
 
         # 包没有安装或版本不符合要求
         install_command = package_spec if required_version else package
-        subprocess.check_call([sys.executable, "-m", "pip", "install", install_command])
-        logger.debug(f"'{install_command}'已安装/更新。")
-        return importlib_metadata.version(package)
+        cmd = [sys.executable, "-m", "pip", "install", install_command]
+        if action == "raise":
+            raise importlib_metadata.PackageNotFoundError(f"依赖包不符合要求, 请使用以下命令安装: {' '.join(cmd)}")
+        else:
+            logger.debug(f"依赖包不符合要求, 自动修正, 命令: {' '.join(cmd)}")
+            subprocess.check_call(cmd)
+            return importlib_metadata.version(package)
 
 
 def invoke(func, args=None, kwargs: dict = None, annotations: dict = None, namespace: dict = None, ignore: list = None):
@@ -243,7 +255,7 @@ def iterable(
         return [_object]
 
 
-def single(_object, default=None):
+def first(_object, default=None):
     """
     将元素变为可迭代对象后, 获取其第一个元素
 
@@ -323,6 +335,14 @@ def get_simple_stack(e):
     return formatted_trace
 
 
+def get_pretty_stack(e: Exception):
+    """
+    获取 stack 信息
+
+    """
+    return "".join(list(exec_formatter.format_exception(e.__class__, e, sys.exc_info()[2])))
+
+
 def clean_rows(*rows: dict, **layout):
     """
     清洗数据
@@ -337,8 +357,8 @@ def clean_rows(*rows: dict, **layout):
     """
 
     def _rename(rule: dict, data: dict):
-        for oldname, newname in rule.items():
-            data[newname] = data.pop(oldname, None)
+        for old_name, new_name in rule.items():
+            data[new_name] = data.pop(old_name, None)
 
     def _show(rule: dict, data: dict):
         for key, flag in rule.items():
@@ -440,9 +460,9 @@ def with_metaclass(
 
                 for func in iterable(autonomous):
                     if isinstance(func, str):
-                        func = getattr(ins, func)
+                        func = getattr(ins, func, None)
 
-                    invoke(func, namespace={"self": ins})
+                    func and invoke(func, namespace={"self": ins})
 
                 return ins
 
@@ -462,13 +482,25 @@ class Method:
 
         return inner
 
-    def install(self):
-        print('install...')
 
+def guess(_object: Any) -> Any:
+    """
+    智能的转换类型
 
-if __name__ == '__main__':
-    # print(Method("sss"))
-    # print(Method("sss"))
-    # print(Method("sss"))
-    # print(Method("sss"))
-    print(inspect.signature(Method))
+    :param _object:
+    :return:
+    """
+    if isinstance(_object, dict):
+        return {k: guess(v) for k, v in _object.items()}
+
+    elif isinstance(_object, (list, tuple, set)):
+        return _object.__class__([guess(i) for i in _object])
+
+    elif isinstance(_object, str):
+        try:
+            return ast.literal_eval(_object)
+        except:  # noqa
+            return _object
+
+    else:
+        return _object
